#include "component_player_control.h"

void _component_player_control_movement_tick(ComponentPlayerControl* self, ECS* ecs);
void _component_player_control_movement(ComponentPlayerControl* self, ECS* ecs);
void _component_player_control_lunge(ComponentPlayerControl* self, ECS* ecs);
void _component_player_control_lunge_tick(ComponentPlayerControl* self, ECS* ecs);

static vec2 SIZE = {64.0f, 64.0f};
static vec3 POSITION = {0.0f, 0.0f, 0.0f};

void
component_player_control_add(ComponentPlayerControl* self, ECS* ecs)
{
	ecs_components_add(ecs, COMPONENT_PLAYER_CONTROL_DEPENDENCIES, COMPONENT_PLAYER_CONTROL_DEPENDENCY_COUNT, self->component.id);

	self->lungeTimerID = entity_timer_add(ecs, COMPONENT_PLAYER_CONTROL_LUNGE_TIMER_MAX);
}

void
component_player_control_delete(ComponentPlayerControl* self, ECS* ecs)
{
	ecs_entity_delete(ecs, self->lungeTimerID);
}

void
_component_player_control_movement(ComponentPlayerControl* self, ECS* ecs)
{
	ComponentPhysics* physics;
	ComponentWorldObject* worldObject;
	vec2 mousePosition;
	vec2 physicsPosition;
	f32 velocityGain;
	f32 distanceFromMouse;

	physics = ecs_component_get(ecs, COMPONENT_PHYSICS, self->component.id);
	worldObject = ecs_component_get(ecs, COMPONENT_WORLD_OBJECT, self->component.id);

	physicsPosition[0] = physics->position[0];
	physicsPosition[1] = physics->position[1];

	mouse_buffer_position_get(&ecs->input->mouse, RENDERER_BUFFER_WORLD, ecs->renderer, mousePosition);

	worldObject->angle = mouse_angle_from_buffer_position_get(&ecs->input->mouse, RENDERER_BUFFER_WORLD, ecs->renderer, physicsPosition);
	
	velocityGain = COMPONENT_PLAYER_CONTROL_VELOCITY_DEFAULT;

	distanceFromMouse = DISTANCE_2D
	(
		physicsPosition[0],
		mousePosition[0],
		physicsPosition[1],
		mousePosition[1]
	);

	if (distanceFromMouse < COMPONENT_PLAYER_CONTROL_VELOCITY_DISTANCE_MAX)
	{
		f32 nextPositionDistance;
		velocityGain *= (distanceFromMouse / COMPONENT_PLAYER_CONTROL_VELOCITY_DISTANCE_MAX);

		nextPositionDistance = component_physics_next_position_distance_get(physics, ecs);

		if (nextPositionDistance < COMPONENT_PLAYER_CONTROL_VELOCITY_STOP_THRESHOLD)
		{
			velocityGain = 0.0f;
			glm_vec2_zero(physics->velocity);
		}
	}

	physics->velocity[0] += velocityGain * cos(worldObject->angle);
	physics->velocity[1] += velocityGain * sin(worldObject->angle);
}

void
_component_player_control_movement_tick(ComponentPlayerControl* self, ECS* ecs)
{
	ComponentWorldObject* worldObject;

	worldObject = ecs_component_get(ecs, COMPONENT_WORLD_OBJECT, self->component.id);

	if (!worldObject->isAirborne)
		_component_player_control_movement(self, ecs);
}

void
_component_player_control_lunge(ComponentPlayerControl* self, ECS* ecs)
{
	ComponentPhysics* physics;
	ComponentWorldObject* worldObject;
	vec2 physicsPosition;

	physics = ecs_component_get(ecs, COMPONENT_PHYSICS, self->component.id);
	worldObject = ecs_component_get(ecs, COMPONENT_WORLD_OBJECT, self->component.id);

	physicsPosition[0] = physics->position[0];
	physicsPosition[1] = physics->position[1];

	worldObject->angle = mouse_angle_from_buffer_position_get(&ecs->input->mouse, RENDERER_BUFFER_WORLD, ecs->renderer, physicsPosition);
	
	physics->velocity[0] = COMPONENT_PLAYER_CONTROL_LUNGE_VELOCITY * cos(worldObject->angle);
	physics->velocity[1] = COMPONENT_PLAYER_CONTROL_LUNGE_VELOCITY * sin(worldObject->angle);
	
	worldObject->heightOffsetVelocity += COMPONENT_PLAYER_CONTROL_LUNGE_HEIGHT_OFFSET_VELOCITY;

	worldObject->isAirborne = true;

	worldObject->heightOffset += COMPONENT_PLAYER_CONTROL_LUNGE_HEIGHT_OFFSET_VELOCITY;
}

void
_component_player_control_lunge_tick(ComponentPlayerControl* self, ECS* ecs)
{
	self->isLunge = false;

	if (control_pressed(ecs->control, CONTROL_ACTION_SECONDARY))
	{
		ComponentTimer* lungeTimer;
		ComponentStamina* stamina;
		ComponentWorldObject* worldObject;

		lungeTimer = ecs_component_get(ecs, COMPONENT_TIMER, self->lungeTimerID);
		stamina = ecs_component_get(ecs, COMPONENT_STAMINA, self->component.id);
		worldObject = ecs_component_get(ecs, COMPONENT_WORLD_OBJECT, self->component.id);

		if 
		(
			lungeTimer->isFinished &&
			stamina->value >= COMPONENT_PLAYER_CONTROL_LUNGE_STAMINA_COST &&
			!worldObject->isAirborne
		)
		{
			component_timer_init(lungeTimer, ecs, COMPONENT_PLAYER_CONTROL_LUNGE_TIMER_MAX);

			stamina->value -= COMPONENT_PLAYER_CONTROL_LUNGE_STAMINA_COST;
			self->isLunge = true;
		}

	}

	if (self->isLunge)
		_component_player_control_lunge(self, ecs);
}

void
component_player_control_tick(ComponentPlayerControl* self, ECS* ecs)
{
	_component_player_control_movement_tick(self, ecs);
	_component_player_control_lunge_tick(self, ecs);
}
