#include "component_world_object.h"

void _component_world_object_shadow_set(ComponentWorldObject* self, ECS* ecs);
void _component_world_object_height_offset_set(ComponentWorldObject* self, ECS* ecs);
void _component_world_object_depth_set(ComponentWorldObject* self, ECS* ecs);
void _component_world_object_circle_collider_set(ComponentWorldObject* self, ECS* ecs);
void _component_world_object_direction_set(ComponentWorldObject* self, ECS* ecs);

void
_component_world_object_shadow_set(ComponentWorldObject* self, ECS* ecs)
{
	ComponentTextureQuad* shadowTextureQuad;
	ComponentTextureQuad* textureQuad;
	ComponentCircleCollider* circleCollider;
	ComponentPhysics* physics;
	f32 shadowScale;

	physics = ecs_component_get(ecs, COMPONENT_PHYSICS, self->component.id);
	circleCollider = ecs_component_get(ecs, COMPONENT_CIRCLE_COLLIDER, self->component.id);
	textureQuad = ecs_component_get(ecs, COMPONENT_TEXTURE_QUAD, self->component.id);
	shadowTextureQuad = ecs_component_get(ecs, COMPONENT_TEXTURE_QUAD, self->shadowID);

	shadowTextureQuad->size[0] = circleCollider->circle.radius;
	shadowTextureQuad->size[1] = COMPONENT_WORLD_OBJECT_SHADOW_HEIGHT;

	shadowTextureQuad->position[0] = physics->position[0];
	shadowTextureQuad->position[1] = physics->position[1] + (textureQuad->size[1] / 2.0f);
	shadowTextureQuad->position[2] = physics->position[2] + COMPONENT_WORLD_OBJECT_SHADOW_Z_OFFSET; 

	shadowScale = 1 - (self->heightOffset * COMPONENT_WORLD_OBJECT_HEIGHT_OFFSET_SHADOW_SCALE_MULTIPLIER);

	shadowTextureQuad->scale[0] = shadowScale;
	shadowTextureQuad->scale[1] = shadowScale;

	shadowTextureQuad->scale[0] = MIN(shadowTextureQuad->scale[0], 0);
	shadowTextureQuad->scale[1] = MIN(shadowTextureQuad->scale[1], 0);
}

void
_component_world_object_height_offset_set(ComponentWorldObject* self, ECS* ecs)
{
	ComponentTextureQuad* textureQuad;

	textureQuad = ecs_component_get(ecs, COMPONENT_TEXTURE_QUAD, self->component.id);

	if (self->isGravityAffected)
		self->heightOffsetVelocity -= COMPONENT_WORLD_OBJECT_GRAVITY;

	self->heightOffset += self->heightOffsetVelocity;

	if (self->heightOffset <= 0.0f)
	{
		self->isAirborne = false;
		self->heightOffset = 0.0f;
		self->heightOffsetVelocity = 0.0f;
	}
	else
		self->isAirborne = true;

	textureQuad->offset[1] = -self->heightOffset;
}

void
_component_world_object_circle_collider_set(ComponentWorldObject* self, ECS* ecs)
{
	ComponentCircleCollider* circleCollider;
	ComponentTextureQuad* textureQuad;
	ComponentPhysics* physics;

	physics = ecs_component_get(ecs, COMPONENT_PHYSICS, self->component.id);
	textureQuad = ecs_component_get(ecs, COMPONENT_TEXTURE_QUAD, self->component.id);
	circleCollider = ecs_component_get(ecs, COMPONENT_CIRCLE_COLLIDER, self->component.id);

	circleCollider->circle.position[0] = physics->position[0];
	circleCollider->circle.position[1] = physics->position[1] + (textureQuad->size[1] / 2.0f);
}

void
_component_world_object_depth_set(ComponentWorldObject* self, ECS* ecs)
{
	ComponentPhysics* physics;
	ComponentTextureQuad* textureQuad;
	f32 depth;
	f32 yOffset;

	physics = ecs_component_get(ecs, COMPONENT_PHYSICS, self->component.id);
	textureQuad = ecs_component_get(ecs, COMPONENT_TEXTURE_QUAD, self->component.id);

	yOffset = COMPONENT_WORLD_OBJECT_DEPTH_Y_OFFSET_MULTIPLIER * ((physics->position[1] + textureQuad->size[1]) / COMPONENT_WORLD_OBJECT_BOUNDS[3]);

	depth = COMPONENT_WORLD_OBJECT_DEPTH_MAX - yOffset;
	depth += COMPONENT_WORLD_OBJECT_DEPTH_MIN;

	physics->position[2] = depth;
}

void
_component_world_object_bounds_set(ComponentWorldObject* self, ECS* ecs)
{
	ComponentPhysics* physics;

	physics = ecs_component_get(ecs, COMPONENT_PHYSICS, self->component.id);

	physics->position[0] = CLAMP(physics->position[0], COMPONENT_WORLD_OBJECT_BOUNDS[0], COMPONENT_WORLD_OBJECT_BOUNDS[2]);
	physics->position[1] = CLAMP(physics->position[1], COMPONENT_WORLD_OBJECT_BOUNDS[1], COMPONENT_WORLD_OBJECT_BOUNDS[3]);
}


void
_component_world_object_direction_set(ComponentWorldObject* self, ECS* ecs)
{
	if 
	(
		(self->angle >= 0.0f && self->angle < PI_FOURTH) ||
		(self->angle >= PI + PI_HALF + PI_FOURTH && self->angle < TAU)
	)
		self->direction = DIRECTION_RIGHT;

	if (self->angle >= PI_FOURTH && self->angle < PI_HALF + PI_FOURTH)
		self->direction = DIRECTION_DOWN;

	if (self->angle >= PI_HALF + PI_FOURTH && self->angle < PI + PI_FOURTH)
		self->direction = DIRECTION_LEFT;

	if (self->angle >= PI + PI_FOURTH && self->angle < PI + PI_HALF + PI_FOURTH)
		self->direction = DIRECTION_UP;
}

void
component_world_object_init
(
	ComponentWorldObject* self, 
	ECS* ecs,
	Texture* texture, 
	vec2 size, 
	vec3 position,
	f32 heightOffset,
	f32 radius,
	bool isShadow
)
{
	component_game_object_init
	(
		ecs_component_get(ecs, COMPONENT_GAME_OBJECT, self->component.id),
		ecs, 
		texture, 
		COMPONENT_WORLD_OBJECT_BUFFER,
		COMPONENT_WORLD_OBJECT_ORIGIN,
		size, 
		position
	);

	component_circle_collider_init
	(
		ecs_component_get(ecs, COMPONENT_CIRCLE_COLLIDER, self->component.id),
		(f32*)position,
		radius
	);

	component_physics_init
	(
		ecs_component_get(ecs, COMPONENT_PHYSICS, self->component.id),
		ecs,
		position,
		COMPONENT_WORLD_OBJECT_VELOCITY_MAX,
		COMPONENT_WORLD_OBJECT_FRICTION
	);

	self->isShadow = isShadow;

	if (self->isShadow)
	{
		self->shadowID = entity_sprite_add
		(
			ecs,
			&ecs->resources->textures[TEXTURE_SHADOW],
			COMPONENT_WORLD_OBJECT_BUFFER,
			COMPONENT_WORLD_OBJECT_ORIGIN,
			(f32*)size,
			(f32*)position
		);

		_component_world_object_shadow_set(self, ecs);
	}

	
	self->heightOffset = heightOffset;
	self->isGravityAffected = true;

	_component_world_object_height_offset_set(self, ecs);
	_component_world_object_circle_collider_set(self, ecs);
}

void
component_world_object_add(ComponentWorldObject* self, ECS* ecs)
{
	ecs_components_add
	(
		ecs, 
		COMPONENT_WORLD_OBJECT_DEPENDENCIES, 
		COMPONENT_WORLD_OBJECT_DEPENDENCY_COUNT, 
		self->component.id
	);
}

void
component_world_object_delete(ComponentWorldObject* self, ECS* ecs)
{
	ecs_entity_delete(ecs, self->shadowID);
}

void
component_world_object_tick(ComponentWorldObject* self, ECS* ecs)
{
	_component_world_object_depth_set(self, ecs);

	if (self->isShadow)
		_component_world_object_shadow_set(self, ecs);
	
	_component_world_object_height_offset_set(self, ecs);
	_component_world_object_bounds_set(self, ecs);
	_component_world_object_direction_set(self, ecs);
	_component_world_object_circle_collider_set(self, ecs);
}
